;---------------------------------------------------------------------------]
;-- TipDay 1.0
;-- Author : Prof. DrAcULA
;-- Date : 5/31/2005 10:01PM
;-- Description : This DLL exports a function named 'ShowTipDay', that
;--               lets you display a 'Tip Of The Day'-window. Useful
;--               for developerz and fun-coderz alike. Enjoy!!!
;--               Inspired by 'mottos' by budyn.
;---------------------------------------------------------------------------]

.386 
.model flat, stdcall
option casemap:none

;---------------------------------------------------------------------------]
;--includes and libs for TipDay
        include         \masm32\include\windows.inc
        include         \masm32\include\user32.inc
        include         \masm32\include\kernel32.inc
        include         \masm32\include\gdi32.inc
        includelib      \masm32\lib\user32.lib
        includelib      \masm32\lib\kernel32.lib
        includelib      \masm32\lib\gdi32.lib
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--local function prototypes
        WndProc         PROTO   :DWORD, :DWORD, :DWORD, :DWORD
        GenerateRandom  PROTO
        FindString      PROTO
        PrevTip         PROTO
        NextTip         PROTO
        ShowTipDay      PROTO STDCALL   :DWORD, :DWORD;--this function'll be exported
;---------------------------------------------------------------------------]


.DATA
;---------------------------------------------------------------------------]
;--initialized data, mostly ZTS
        szError         DB      "Error",0
        szFileNotFound  DB      "Tips file not found or is in use!",0
        szAbout         DB      13,10
                        DB      "TipDay v1.0",13,10
                        DB      "Coded by Prof. DrAcULA",13,10,13,10
                        DB      "on May 31, 2005.",13,10,13,10
                        DB      "All rights reserved",13,10,13,10
                        DB      "Greetz & thx : ",13,10
                        DB      "'budyn' for original-concept, ",13,10
                        DB      "My Girly :)",13,10
                        DB      "and, all assembly coderz!", 0
        Random          DB      00
        Tag             DD      12345678
        SizeRW          DD      0
        AddrBuff        DD      246 DUP(0)
        TextAmount      DB      1 DUP(0)
        TextCount       DB      1 DUP(0)
        szFilename      DB      "tips.dat",0
;---------------------------------------------------------------------------]


.DATA?
;---------------------------------------------------------------------------]
;--uninitialized data
        hInst           DD      ?       ;--instance handle
        hFile           DD      ?       ;--file handle
        hEdit           DD      ?       ;--editbox handle
        hMemory         DD      ?       ;--memory handle
        pMemory         DD      ?       ;--pointer to memory
        hStatic         DD      ?       ;--static handle
        TipModel        DD      ?       ;--Model of tip-window to use
        Startup         DD      ?       ;--Startup checkbox status
        ReturnValue     DD      ?       ;--Function return value
        IsAbout         DB      ?       ;--User clicked About-bitmap?
;---------------------------------------------------------------------------]


.CONST
;---------------------------------------------------------------------------]
;--constants
        ID_DIALOG       EQU     100     ;--window
        ID_STATIC       EQU     101     ;--static
        ID_BITMAP       EQU     102     ;--bitmap
        ID_EDIT         EQU     103     ;--editbox
        ID_PREV         EQU     104     ;--previous button
        ID_NEXT         EQU     105     ;--next button
        ID_EXIT         EQU     106     ;--exit button
        ID_STARTUP      EQU     107     ;--startup checkbox
        MEMSIZE         EQU     65535   ;--memory size to allocate
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--macros
        RGB     MACRO   Red, Green, Blue
                mov     eax, Red or (Green SHL 8) or (Blue SHL 16)
        ENDM
;---------------------------------------------------------------------------]


.CODE
;---------------------------------------------------------------------------]
;--this function is called when DLL is loaded.
DllEntry        PROC    hInstance :DWORD,reason :DWORD,reservedl :DWORD
        mov     eax, hInstance
        mov     hInst, eax      ;--store instance handle in hInst
        mov     eax, 1
        ret
DllEntry        ENDP
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--this is the function to be exported.
;       Arguments: 
;                  (1). TipMdl > Accepts either 0 or 1
;                  If 0, a white-window with white bitmap and grey-text is
;                  is shown. If 1, a black-window with black-bitmap and
;                  light-yellow text is shown.
;                  (2). Strtup > Accepts either 0 or 1
;                  If 0 'Show-at-startup' checkbox is checked, else
;                  the checkbox is not checked.
;       Return Value:
;                  If user unchecked 'Show tips at startup'-checkbox
;                  then return value is 1 else 0. This value is returned
;                  in eax.
;---------------------------------------------------------------------------]
ShowTipDay      PROC STDCALL    TipMdl:DWORD, Strtup:DWORD
        mov     IsAbout, 0      ;--User has'nt clicked About-bitmap
        mov     eax, TipMdl     ;--store TipMdl in eax
        mov     TipModel, eax   ;--store value of eax in TipModel
        mov     eax, Strtup     ;--store Strtup in eax
        mov     Startup, eax    ;--store value of eax in Startup
        invoke  DialogBoxParam, hInst, ID_DIALOG, 0, offset WndProc, 0  ;--show window
        mov     eax, ReturnValue
        ret
ShowTipDay      ENDP
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--our Window Procedure
WndProc         PROC    hWnd:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
        mov     eax, uMsg               ;--mov window-message to eax
        cmp     eax, WM_INITDIALOG      ;--if its initialization
        jz      _INITDIALOG             ;--jump to prodecure
        cmp     eax, WM_CTLCOLORSTATIC  ;--if its colorstatic
        jz      _CTLCOLORSTATIC         ;--goto procedure
        cmp     eax, WM_COMMAND         ;--if its command
        jz      _COMMAND                ;--jump to procedure
        cmp     eax, WM_CLOSE           ;--if its close
        jz      _CLOSE                  ;--jump to procedure
        
  _DEFAULT:
        xor     eax, eax
        ret
        
  _INITDIALOG:
        .if Startup==0  ;--Check 'Show tips at startup'-checkbox
                invoke  CheckDlgButton, hWnd, ID_STARTUP, BST_CHECKED
        .endif                
        .if TipModel==1 ;--Load Black-bitmap
                invoke  LoadBitmap, hInst, 111
        .else           ;--Load White-bitmap
                invoke  LoadBitmap, hInst, 222
        .endif                
        invoke  SendDlgItemMessage, hWnd, ID_BITMAP, STM_SETIMAGE, IMAGE_BITMAP, eax
        invoke  GetDlgItem, hWnd, ID_EDIT       ;--get handle of Editbox
        mov     hEdit, eax                      ;--store it in hEdit
        invoke  GetDlgItem, hWnd, ID_STATIC     ;--get handle of static
        mov     hStatic, eax                    ;--store it in hStatic
        invoke  CreateFileA, offset szFilename, GENERIC_READ, 0, 0, OPEN_EXISTING, \
                FILE_ATTRIBUTE_READONLY, 0
        cmp     eax, -1                         ;--if file not found
        jz      _NoFile                         ;--goto _NoFile
        mov     hFile, eax                      ;--move handle of file in hFile
        invoke  GlobalAlloc, GMEM_MOVEABLE or GMEM_ZEROINIT, MEMSIZE    ;--allocate buffer
        mov     hMemory, eax                    ;--store handle to buffer in hMemory
        invoke  GlobalLock, eax                 ;--lock buffer
        mov     pMemory, eax                    ;--move pointer to buffer in pMemory
        invoke  ReadFile, hFile, pMemory, MEMSIZE-1, offset SizeRW, 0   ;--read file
        invoke  FindString                      ;--find tip
        invoke  GenerateRandom                  ;--get random tip
        sub     ecx, ecx
        mov     cl, [Random]
        mov     [TextCount], cl
        mov     esi, [AddrBuff]+ecx*4
        inc     esi
        invoke  SendMessageA, hEdit, WM_SETTEXT, 0, esi ;--set tip in editbox
        
  _MarchOn:        
        invoke  SendDlgItemMessage, hWnd, ID_EDIT, WM_SETFOCUS, 0, 0    ;--set editbox if focus
        invoke  HideCaret, hEdit        ;--hide caret in editbox
        ret
        
  _CTLCOLORSTATIC:
        mov     eax, hEdit
        cmp     lParam, eax
        je      _ColorEdit
        mov     eax, hStatic
        cmp     lParam, eax
        jne     _DEFAULT
        .if TipModel==1                 ;--if Black-model
                RGB     0, 0, 0         ;--black
        .else                           ;--if White-model
                RGB     255, 255, 255   ;--white
        .endif
        invoke  SetBkColor, wParam, eax ;--set background color(of static)
        jmp     _Brush
        
  _ColorEdit:
        .if TipModel==1                 ;--if Black-model
                RGB     0, 0 , 0        ;--black
        .else                           ;--if White-model
                RGB     255, 255, 255   ;--white
        .endif
        invoke  SetBkColor, wParam, eax ;--set backcolor of editbox
        .if TipModel==1                 ;--if Black-model
                RGB     255, 255, 160   ;--light yellow
        .else                           ;--if White-model
                RGB     80, 80, 80      ;--dark grey
        .endif                
        invoke  SetTextColor, wParam, eax       ;--set text-color of editbox
        
  _Brush:
        .if TipModel==1                 ;--if Blackmodel
                RGB     0, 0 , 0        ;--then black
        .else                           ;--else
                RGB     255, 255, 255   ;--white
        .endif
        invoke  CreateSolidBrush, eax   ;--create brush of black/white color
        ret
        
  _CLOSE:
        invoke  IsDlgButtonChecked, hWnd, ID_STARTUP    ;--is 'Show tips at startup' checked
        .if eax==BST_CHECKED                            ;--if it is checked
                mov     ReturnValue, 0                  ;--ReturnValue is 0
        .else                                           ;--otherwise
                mov     ReturnValue, 1                  ;--ReturnValue is 1
        .endif      
        invoke  CloseHandle, hFile                      ;--close file
        invoke  GlobalUnlock, pMemory                   ;--unlock buffer
        invoke  GlobalFree, hMemory                     ;--free buffer
        invoke  EndDialog, hWnd, 0                      ;--close dialog
        xor     eax, eax
        ret
        
  _COMMAND:
        mov     eax, wParam             ;--mov wParam to eax
        cmp     ax, ID_NEXT             ;--if Next-button is clicked
        jz      _COMMAND_NEXTTIP        ;--jump to procedure
        cmp     ax, ID_PREV             ;--if Prev-button is clicked
        jz      _COMMAND_PREVTIP        ;--jump to procedure
        cmp     ax, ID_BITMAP           ;--if bitmap is clicked
        jz      _COMMAND_ABOUT          ;--jump to routine
        cmp     ax, ID_EXIT             ;--if Exit-button is clicked
        jz      _CLOSE                  ;--jump to procedure
        xor     eax, eax
        ret
        
  _COMMAND_NEXTTIP:
        .if IsAbout==1  ;--if user has clicked About-bitmap
                .if TipModel==1         ;--if model is black
                        invoke  LoadBitmap, hInst, 111  ;--load black Tip-bitmap
                .else                                   ;--otherwise
                        invoke  LoadBitmap, hInst, 222  ;--load white Tip-bitmap
                .endif
                invoke  SendDlgItemMessage, hWnd, ID_BITMAP, STM_SETIMAGE, IMAGE_BITMAP, eax
                mov     IsAbout, 0      ;--set IsAbout to false
        .endif
        invoke  NextTip
        xor     eax, eax
        ret
        
  _COMMAND_PREVTIP:
        .if IsAbout==1  ;--if user has clicked About-bitmap
                .if TipModel==1         ;--if model is black
                        invoke  LoadBitmap, hInst, 111  ;--load black Tip-bitmap
                .else                                   ;--otherwise
                        invoke  LoadBitmap, hInst, 222  ;--load white Tip-bitmap
                .endif
                invoke  SendDlgItemMessage, hWnd, ID_BITMAP, STM_SETIMAGE, IMAGE_BITMAP, eax
                mov     IsAbout, 0      ;--set IsAbout to false
        .endif
        invoke  PrevTip
        xor     eax, eax
        ret
        
  _COMMAND_ABOUT:
        mov     IsAbout, 1      ;--set IsAbout to true
        .if TipModel==1         ;--if model is black
                invoke  LoadBitmap, hInst, 333  ;--load black About-bitmap
        .else                                   ;--else
                invoke  LoadBitmap, hInst, 444  ;--load white About-bitmap
        .endif                
        invoke  SendDlgItemMessage, hWnd, ID_BITMAP, STM_SETIMAGE, IMAGE_BITMAP, eax
        mov     esi, offset szAbout
        invoke  SendMessageA, hEdit, WM_SETTEXT, 0, esi ;--set about text to editbox
        xor     eax, eax
        ret
        
  _NoFile:
        invoke  MessageBoxA, hWnd, offset szFileNotFound, offset szError, MB_OK or MB_ICONEXCLAMATION
        jmp     _MarchOn
WndProc         ENDP
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--Finds a tip
FindString      PROC
        pushad
        mov     esi, pMemory
        mov     al, 3Bh
        sub     ebx, ebx
        sub     ecx, ecx
  _FindNext:
        cmp     al, [esi]
        je      _SaveAddr
        cmp     bl, [esi]
        je      _End
        inc     esi
        jmp     _FindNext
  _SaveAddr:
        mov     [AddrBuff]+ecx*4, esi
        inc     ecx
        cmp     ecx, 01
        jle     _NextOne
        mov     [esi-1], bl
  _NextOne:
        inc     esi
        jmp     _FindNext
  _End:
        mov     [TextAmount], cl
        dec     [TextAmount]
        popad
        ret
FindString      ENDP
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--Displays next tip
NextTip         PROC
        pushad
        sub     ecx, ecx
        mov     cl, TextCount
        inc     cl
        cmp     cl, TextAmount
        jge     _EndNextTip
        mov     esi, [AddrBuff]+ecx*4
        inc     esi
        invoke  SendMessageA, hEdit, WM_SETTEXT, 0, esi
        inc     TextCount
  _EndNextTip:
        popad
        ret
NextTip         ENDP
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--displays previous tip
PrevTip         PROC
        pushad
        sub     ecx, ecx
        mov     cl, TextCount
        cmp     cl, 1
        jle     _EndPrevTip
        dec     cl
        dec     cl
        lea     esi, AddrBuff[ecx*4]
        mov     esi, [esi]
        inc     esi
        invoke  SendMessageA, hEdit, WM_SETTEXT, 0, esi
        dec     TextCount
  _EndPrevTip:
        popad
        ret
PrevTip         ENDP
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--Generate random tip-position
GenerateRandom  PROC
        pushad
        mov     edi, offset Random
        mov     ecx, 08
  _GenLoop:
        invoke  GetTickCount
        shl     eax, cl
        xor     eax, [Tag]
        shr     eax, cl
        mov     [Tag], eax
        and     eax, 0000000Fh
        mov     dl, [TextAmount]
        cmp     al, dl
        jge     _GenLoop
        stosb
        popad
        ret
GenerateRandom  ENDP
;---------------------------------------------------------------------------]

END     DllEntry
;---------------------------------------------------------------------------]
;-- 2005, Prof. DrAcULA : "Knowledge is the sweetest blood!"
;---------------------------------------------------------------------------]